"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
/* eslint-disable @typescript-eslint/no-use-before-define */
const xcm_config_1 = require("@moonbeam-network/xcm-config");
const util_1 = require("@polkadot/util");
const XTokensContract_1 = require("../contracts/XTokensContract");
const polkadot_1 = require("../polkadot");
const polkadot_utils_1 = require("../polkadot/polkadot.utils");
const sdk_utils_1 = require("./sdk.utils");
async function create(options) {
    return {
        moonbase: await createChainSdk(xcm_config_1.moonbase, options),
        moonbeam: await createChainSdk(xcm_config_1.moonbeam, options),
        moonriver: await createChainSdk(xcm_config_1.moonriver, options),
    };
}
exports.create = create;
async function createChainSdk(configGetter, options) {
    const contract = new XTokensContract_1.XTokensContract(options.ethersSigner);
    return {
        asset: configGetter.asset,
        chain: configGetter.chain,
        subscribeToAssetsBalanceInfo: async (account, cb) => {
            let lastBalance = 0n;
            let lastInfo = [];
            const polkadot = await polkadot_1.PolkadotService.create(configGetter.chain.ws);
            const handler = (data) => {
                const isBalance = typeof data === 'bigint';
                lastBalance = isBalance ? data : lastBalance;
                lastInfo = (isBalance ? lastInfo : data)
                    .map((info) => {
                    if (info.asset.isNative) {
                        // eslint-disable-next-line no-param-reassign
                        info.balance = lastBalance;
                    }
                    return info;
                })
                    .sort(polkadot_utils_1.sortByBalanceAndChainName);
                cb(lastInfo);
            };
            const unsubscribeBalance = await polkadot.subscribeToBalance(account, handler);
            const unsubscribeInfo = await polkadot.subscribeToAssetsBalanceInfo(account, configGetter, handler);
            return () => {
                unsubscribeBalance();
                unsubscribeInfo();
            };
        },
        deposit: (asset) => {
            const { chains, from } = configGetter.deposit(asset);
            return {
                chains,
                from: (chain) => {
                    const { asset: assetConfig, origin, config } = from(chain);
                    return {
                        get: async (account, sourceAccount, primaryAccount) => {
                            const polkadot = await polkadot_1.PolkadotService.create(configGetter.chain.ws);
                            const foreignPolkadot = await polkadot_1.PolkadotService.create(config.origin.ws);
                            const meta = foreignPolkadot.getMetadata();
                            const nativeAsset = configGetter.assets[meta.symbol];
                            const createExtrinsic = (0, sdk_utils_1.getCreateExtrinsic)({
                                account,
                                config,
                                foreignPolkadot,
                                nativeAsset,
                                polkadot,
                                primaryAccount,
                            });
                            const [decimals, sourceBalance, existentialDeposit, sourceFeeBalance, moonChainFee, sourceMinBalance = 0n, min,] = await Promise.all([
                                polkadot.getAssetDecimals(assetConfig.id),
                                foreignPolkadot.getGenericBalance(primaryAccount || sourceAccount, config.balance),
                                foreignPolkadot.getExistentialDeposit(),
                                config.sourceFeeBalance
                                    ? foreignPolkadot.getGenericBalance(sourceAccount, config.sourceFeeBalance)
                                    : undefined,
                                config.isNativeAssetPayingMoonFee
                                    ? polkadot.getAssetFee(nativeAsset.id, config.origin.weight)
                                    : undefined,
                                config.sourceMinBalance
                                    ? foreignPolkadot.getAssetMinBalance(config.sourceMinBalance)
                                    : undefined,
                                assetConfig.isNative
                                    ? polkadot_1.PolkadotService.getChainMin(config.origin.weight, configGetter.chain.unitsPerSecond)
                                    : polkadot.getAssetFee(assetConfig.id, config.origin.weight),
                            ]);
                            return {
                                asset: { ...assetConfig, decimals },
                                existentialDeposit,
                                min,
                                moonChainFee,
                                native: { ...nativeAsset, decimals: meta.decimals },
                                origin,
                                source: config.origin,
                                sourceBalance,
                                sourceFeeBalance: !(0, util_1.isUndefined)(sourceFeeBalance)
                                    ? { ...meta, balance: sourceFeeBalance }
                                    : undefined,
                                sourceMinBalance,
                                getFee: async (amount = sourceBalance) => {
                                    const extrinsic = await createExtrinsic(amount);
                                    const info = await extrinsic.paymentInfo(sourceAccount);
                                    return info.partialFee.toBigInt();
                                },
                                send: async (amount, cb) => {
                                    const extrinsic = await createExtrinsic(amount);
                                    const hash = await extrinsic.signAndSend(sourceAccount, {
                                        signer: options.polkadotSigner,
                                    }, cb && (0, sdk_utils_1.createExtrinsicEventHandler)(config, cb));
                                    return hash.toString();
                                },
                            };
                        },
                    };
                },
            };
        },
        withdraw: (asset) => {
            const { chains, to } = configGetter.withdraw(asset);
            return {
                chains,
                to: (chain) => {
                    const { asset: assetConfig, origin, config } = to(chain);
                    return {
                        get: async (destinationAccount) => {
                            const polkadot = await polkadot_1.PolkadotService.create(configGetter.chain.ws);
                            const destinationPolkadot = await polkadot_1.PolkadotService.create(config.destination.ws);
                            const meta = destinationPolkadot.getMetadata();
                            const nativeAsset = configGetter.assets[meta.symbol];
                            const [decimals, destinationBalance, existentialDeposit] = await Promise.all([
                                assetConfig.isNative
                                    ? configGetter.chain.decimals
                                    : polkadot.getAssetDecimals(assetConfig.id),
                                destinationPolkadot.getGenericBalance(destinationAccount, config.balance),
                                destinationPolkadot.getExistentialDeposit(),
                            ]);
                            const destinationFee = BigInt(config.weight * config.feePerWeight);
                            const min = destinationFee + existentialDeposit;
                            return {
                                asset: { ...assetConfig, decimals },
                                destination: config.destination,
                                destinationBalance,
                                destinationFee,
                                existentialDeposit,
                                min,
                                native: { ...nativeAsset, decimals: meta.decimals },
                                origin,
                                getFee: async (amount) => contract.getTransferFees(destinationAccount, amount, assetConfig, config),
                                send: async (amount, cb) => {
                                    const tx = await contract.transfer(destinationAccount, amount, assetConfig, config);
                                    if (cb) {
                                        (0, sdk_utils_1.createTransactionEventHandler)(options.ethersSigner, tx.hash, cb);
                                    }
                                    return tx.hash;
                                },
                            };
                        },
                    };
                },
            };
        },
    };
}
//# sourceMappingURL=sdk.js.map