/* eslint-disable @typescript-eslint/no-use-before-define */
import { moonbase, moonbeam, moonriver, } from '@moonbeam-network/xcm-config';
import { isUndefined } from '@polkadot/util';
import { XTokensContract } from '../contracts/XTokensContract';
import { PolkadotService } from '../polkadot';
import { sortByBalanceAndChainName } from '../polkadot/polkadot.utils';
import { createExtrinsicEventHandler, createTransactionEventHandler, getCreateExtrinsic, } from './sdk.utils';
export async function create(options) {
    return {
        moonbase: await createChainSdk(moonbase, options),
        moonbeam: await createChainSdk(moonbeam, options),
        moonriver: await createChainSdk(moonriver, options),
    };
}
async function createChainSdk(configGetter, options) {
    const contract = new XTokensContract(options.ethersSigner);
    return {
        asset: configGetter.asset,
        chain: configGetter.chain,
        subscribeToAssetsBalanceInfo: async (account, cb) => {
            let lastBalance = 0n;
            let lastInfo = [];
            const polkadot = await PolkadotService.create(configGetter.chain.ws);
            const handler = (data) => {
                const isBalance = typeof data === 'bigint';
                lastBalance = isBalance ? data : lastBalance;
                lastInfo = (isBalance ? lastInfo : data)
                    .map((info) => {
                    if (info.asset.isNative) {
                        // eslint-disable-next-line no-param-reassign
                        info.balance = lastBalance;
                    }
                    return info;
                })
                    .sort(sortByBalanceAndChainName);
                cb(lastInfo);
            };
            const unsubscribeBalance = await polkadot.subscribeToBalance(account, handler);
            const unsubscribeInfo = await polkadot.subscribeToAssetsBalanceInfo(account, configGetter, handler);
            return () => {
                unsubscribeBalance();
                unsubscribeInfo();
            };
        },
        deposit: (asset) => {
            const { chains, from } = configGetter.deposit(asset);
            return {
                chains,
                from: (chain) => {
                    const { asset: assetConfig, origin, config } = from(chain);
                    return {
                        get: async (account, sourceAccount, primaryAccount) => {
                            const polkadot = await PolkadotService.create(configGetter.chain.ws);
                            const foreignPolkadot = await PolkadotService.create(config.origin.ws);
                            const meta = foreignPolkadot.getMetadata();
                            const nativeAsset = configGetter.assets[meta.symbol];
                            const createExtrinsic = getCreateExtrinsic({
                                account,
                                config,
                                foreignPolkadot,
                                nativeAsset,
                                polkadot,
                                primaryAccount,
                            });
                            const [decimals, sourceBalance, existentialDeposit, sourceFeeBalance, moonChainFee, sourceMinBalance = 0n, min,] = await Promise.all([
                                polkadot.getAssetDecimals(assetConfig.id),
                                foreignPolkadot.getGenericBalance(primaryAccount || sourceAccount, config.balance),
                                foreignPolkadot.getExistentialDeposit(),
                                config.sourceFeeBalance
                                    ? foreignPolkadot.getGenericBalance(sourceAccount, config.sourceFeeBalance)
                                    : undefined,
                                config.isNativeAssetPayingMoonFee
                                    ? polkadot.getAssetFee(nativeAsset.id, config.origin.weight)
                                    : undefined,
                                config.sourceMinBalance
                                    ? foreignPolkadot.getAssetMinBalance(config.sourceMinBalance)
                                    : undefined,
                                assetConfig.isNative
                                    ? PolkadotService.getChainMin(config.origin.weight, configGetter.chain.unitsPerSecond)
                                    : polkadot.getAssetFee(assetConfig.id, config.origin.weight),
                            ]);
                            return {
                                asset: { ...assetConfig, decimals },
                                existentialDeposit,
                                min,
                                moonChainFee,
                                native: { ...nativeAsset, decimals: meta.decimals },
                                origin,
                                source: config.origin,
                                sourceBalance,
                                sourceFeeBalance: !isUndefined(sourceFeeBalance)
                                    ? { ...meta, balance: sourceFeeBalance }
                                    : undefined,
                                sourceMinBalance,
                                getFee: async (amount = sourceBalance) => {
                                    const extrinsic = await createExtrinsic(amount);
                                    const info = await extrinsic.paymentInfo(sourceAccount);
                                    return info.partialFee.toBigInt();
                                },
                                send: async (amount, cb) => {
                                    const extrinsic = await createExtrinsic(amount);
                                    const hash = await extrinsic.signAndSend(sourceAccount, {
                                        signer: options.polkadotSigner,
                                    }, cb && createExtrinsicEventHandler(config, cb));
                                    return hash.toString();
                                },
                            };
                        },
                    };
                },
            };
        },
        withdraw: (asset) => {
            const { chains, to } = configGetter.withdraw(asset);
            return {
                chains,
                to: (chain) => {
                    const { asset: assetConfig, origin, config } = to(chain);
                    return {
                        get: async (destinationAccount) => {
                            const polkadot = await PolkadotService.create(configGetter.chain.ws);
                            const destinationPolkadot = await PolkadotService.create(config.destination.ws);
                            const meta = destinationPolkadot.getMetadata();
                            const nativeAsset = configGetter.assets[meta.symbol];
                            const [decimals, destinationBalance, existentialDeposit] = await Promise.all([
                                assetConfig.isNative
                                    ? configGetter.chain.decimals
                                    : polkadot.getAssetDecimals(assetConfig.id),
                                destinationPolkadot.getGenericBalance(destinationAccount, config.balance),
                                destinationPolkadot.getExistentialDeposit(),
                            ]);
                            const destinationFee = BigInt(config.weight * config.feePerWeight);
                            const min = destinationFee + existentialDeposit;
                            return {
                                asset: { ...assetConfig, decimals },
                                destination: config.destination,
                                destinationBalance,
                                destinationFee,
                                existentialDeposit,
                                min,
                                native: { ...nativeAsset, decimals: meta.decimals },
                                origin,
                                getFee: async (amount) => contract.getTransferFees(destinationAccount, amount, assetConfig, config),
                                send: async (amount, cb) => {
                                    const tx = await contract.transfer(destinationAccount, amount, assetConfig, config);
                                    if (cb) {
                                        createTransactionEventHandler(options.ethersSigner, tx.hash, cb);
                                    }
                                    return tx.hash;
                                },
                            };
                        },
                    };
                },
            };
        },
    };
}
//# sourceMappingURL=sdk.js.map